/*
 *  Copyright (C) 2012 Joos Kiener <Joos.Kiener@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.

 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bitbucket.kienerj.moleculedatabaseframework.entity;

import com.fasterxml.jackson.annotation.JsonIdentityInfo;
import com.fasterxml.jackson.annotation.ObjectIdGenerators;
import com.mysema.query.annotations.QueryInit;
import javax.persistence.AssociationOverride;
import javax.persistence.AssociationOverrides;
import javax.persistence.Column;
import javax.persistence.EmbeddedId;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.Table;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.annotations.Generated;
import org.hibernate.annotations.GenerationTime;

/**
 * <p> A composition consists of a Chemical Structure, a Chemical Compound and
 * the percentage at which the structure is present in the compound. </p>
 *
 * @author Joos Kiener <Joos.Kiener@gmail.com>
 */
@Entity
@Table(name = "compound_composition")
@AssociationOverrides({
    @AssociationOverride(name = "pk.chemicalStructure",
    joinColumns =
    @JoinColumn(name = "chemical_structure_id")),
    @AssociationOverride(name = "pk.compound",
    joinColumns =
    @JoinColumn(name = "chem_compound_id"))})
@JsonIdentityInfo(generator=ObjectIdGenerators.IntSequenceGenerator.class, property="@id")
@Data
@EqualsAndHashCode(callSuper = false, of = {"pk"})
public class ChemicalCompoundComposition extends BaseEntity {

    @EmbeddedId
    @QueryInit({"chemicalStructure", "compound"})
    private ChemicalCompoundCompositionID pk = new ChemicalCompoundCompositionID();
    @Column(name = "percentage")
    private Double percentage;
    @Column(name = "composition_id", columnDefinition="serial")
    @Generated(GenerationTime.INSERT)
    private Long compositionId;

    /**
     * For JPA Provider
     */
    public ChemicalCompoundComposition() {
    }

    public ChemicalCompoundComposition(ChemicalCompound compound,
            ChemicalStructure structure,
            Double percentage) {

        this.percentage = percentage;
        pk.setChemicalStructure(structure);
        pk.setCompound(compound);
    }

    public ChemicalCompound getCompound() {
        return pk.getCompound();
    }

    public void setCompound(ChemicalCompound compound) {
        pk.setCompound(compound);
    }

    public ChemicalStructure getChemicalStructure() {
        return pk.getChemicalStructure();
    }

    public void setChemicalStructure(ChemicalStructure structure) {
        pk.setChemicalStructure(structure);
    }

    @Override
    public Long getId(){
        return this.compositionId;
    }

    public void setId(Long id) {
        this.compositionId = id;
    }
}
