/*
 *  Copyright (C) 2013 Joos Kiener <Joos.Kiener@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.

 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bitbucket.kienerj.moleculedatabaseframework.entity.listener;

import javax.annotation.PostConstruct;
import javax.persistence.PostLoad;
import org.bitbucket.kienerj.moleculedatabaseframework.chemistry.ChemistryFormat;
import org.bitbucket.kienerj.moleculedatabaseframework.entity.ChemicalStructure;
import org.slf4j.ext.XLogger;
import org.slf4j.ext.XLoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * <p>Injects the configured ChemicalStructureFormat into each ChemicalStructure
 * entity</p>
 * <p>See <a href="http://stackoverflow.com/questions/12155632/injecting-a-spring-dependency-into-a-jpa-entitylistener">this stackoverflow thread</a>
 *  and <a href="http://blog-en.lineofsightnet.com/2012/08/dependency-injection-on-stateless-beans.html>the linked solution</a>.</p>
 *
 * @author Joos Kiener <Joos.Kiener@gmail.com>
 */
@Component
public class StructureFormatListener {

    private static final XLogger logger = XLoggerFactory.getXLogger("StructureFormatListener");

    static private ChemistryFormat chemistryFormat;

    @Autowired
    public void setChemistryFormat(ChemistryFormat chemistryFormat){
        this.chemistryFormat = chemistryFormat;
    }

    @PostLoad
    public void setChemistryFormat(ChemicalStructure structure){
        structure.setChemistryFormat(chemistryFormat);
    }

    @PostConstruct
    public void init() {
        logger.info("Initializing with dependency [" + chemistryFormat + "]");
    }
}
