/*
 * Copyright (C) 2013 Joos Kiener <Joos.Kiener@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bitbucket.kienerj.moleculedatabaseframework.io;

import java.text.ParseException;
import java.util.Map;
import org.bitbucket.kienerj.moleculedatabaseframework.entity.ChemicalCompoundContainer;
import org.bitbucket.kienerj.moleculedatabaseframework.entity.Containable;
import org.slf4j.ext.XLogger;
import org.slf4j.ext.XLoggerFactory;

/**
 *
 * @author Joos Kiener <Joos.Kiener@gmail.com>
 */
public class ContainerFactory<T extends ChemicalCompoundContainer> {

    private static final XLogger logger = XLoggerFactory.getXLogger("ContainerFactory");

    private final Class<T> containerClass;

    public ContainerFactory (Class<T> containerClass){
        this.containerClass = containerClass;
    }

    /**
     * <p>
     * Creates a new <code>Container</code> Instance from the given property mappings
     * and property values.
     * </p>
     * <p>
     * This only sets "simple" properties like primitives, Strings or Dates.
     * Associated Containable is not set.
     * </p>
     * <p>
     * </p>
     * @param sdfRecord
     * @param mappedProperties
     * @return
     * @throws ParseException when the given data can not be converted to the target type
     * @throws IllegalAccessException
     * @throws NoSuchFieldException in case of an invalid mapping in <code>mappedProperties</code>
     * @throws InstantiationException if <code>compoundClass</code> does not have a zero argument constructor
     */
    public T createContainer(Containable containable, Map<String, String> mappedProperties,
            Map<String, String> propertyValues) throws ParseException,
            IllegalAccessException, NoSuchFieldException, InstantiationException {

        logger.entry(mappedProperties, propertyValues);
        T container = containerClass.newInstance();
        container.setContainable(containable);
        containable.addChemicalCompoundContainer(container);
        PropertySetter.setProperties(container, containerClass, mappedProperties, propertyValues);
        logger.exit(containable);
        return container;
    }
}
