/*
 *  Copyright (C) 2011 Joos Kiener
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.

 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.bitbucket.kienerj.moleculedatabaseframework.io;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import lombok.Getter;

/**
 * <p>
 * Records the result of an SD-File import like errors occurred, compounds imported
 * and also reports compounds that already existed in the database and were hence
 * not imported again.
 * </p>
 *
 * @author Joos Kiener <Joos.Kiener@gmail.com>
 */
public class EntityImportResult {

    @Getter
    private List<Long> importedIds;
    @Getter
    private List<Long> alreadyInDatabaseIds;
    @Getter
    private List<CompoundImportError> importErrors;

    public EntityImportResult(){

        this.importedIds = Collections.synchronizedList(
                new ArrayList<Long>());
        this.alreadyInDatabaseIds = Collections.synchronizedList(
                new ArrayList<Long>());
        this.importErrors = Collections.synchronizedList(
                new ArrayList<CompoundImportError>());

    }

    public void addImportError(CompoundImportError importError){
        importErrors.add(importError);
    }

    public void addImportErrors(Collection<CompoundImportError> importErrors){
        this.importErrors.addAll(importErrors);
    }

    public void addImportedEntity(Long id){
        importedIds.add(id);
    }
    public void addImportedEntities(Collection<Long > ids){
        importedIds.addAll(ids);
    }

    public void addAlreadyPresentEntity(Long  id){
        alreadyInDatabaseIds.add(id);
    }
    public void addAlreadyPresentEntities(Collection<Long > ids){
        alreadyInDatabaseIds.addAll(ids);
    }
}
