/*
 *  Copyright (C) 2012 Joos Kiener <Joos.Kiener@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.

 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bitbucket.kienerj.moleculedatabaseframework.repository;

import java.util.List;
import org.bitbucket.kienerj.moleculedatabaseframework.entity.ChemicalCompoundComposition;
import org.bitbucket.kienerj.moleculedatabaseframework.entity.ChemicalCompoundCompositionID;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.querydsl.QueryDslPredicateExecutor;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

/**
 * Repository for <code>ChemicalCompoundComposition</code> entity.
 *
 * @author Joos Kiener <Joos.Kiener@gmail.com>
 */
@Transactional(propagation = Propagation.MANDATORY)
@Repository("compositionRepository")
public interface ChemicalCompoundCompositionRepository
        extends JpaRepository<ChemicalCompoundComposition, ChemicalCompoundCompositionID>,
        QueryDslPredicateExecutor<ChemicalCompoundComposition> {

    /**
     * Gets a composition by it unique numeric database key.
     * @param id
     * @return
     */
    ChemicalCompoundComposition findByCompositionId(Long id);

    /**
     * Returns all <code>ChemicalCompoundCompositions</code> a given
     * <code>ChemicalStructure</code> occurs in.
     *
     * @param chemicalStructureId id of the <code>ChemicalStructure</code>
     */
    List<ChemicalCompoundComposition> findByPkChemicalStructureId(Long chemicalStructureId);

    /**
     * Returns all <code>ChemicalCompoundCompositions</code> a given
     * <code>ChemicalCompound</code> occurs in.
     *
     * @param compoundId id of the <code>ChemicalCompound</code>
     */
    List<ChemicalCompoundComposition> findByPkCompoundId(Long compoundId);

    @Modifying(clearAutomatically = true)
    @Query(value="UPDATE compound_composition SET chemical_structure_id = ?2, "
            + "chem_compound_id = ?3, percentage = ?4, version = ?5 + 1 "
            + "WHERE composition_id = ?1 AND version = ?5",
            nativeQuery=true)
    int updateCompoundComposition(long compositionId, long structureId, long compoundId, double percentage, long version);
}
