/*
 *  Copyright (C) 2012 Joos Kiener <Joos.Kiener@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.

 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bitbucket.kienerj.moleculedatabaseframework.repository;

import java.util.List;
import org.bitbucket.kienerj.moleculedatabaseframework.entity.ChemicalCompound;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * Spring Data Repository for <code>ChemicalCompound</code> implementations.
 * </p>
 *
 * @author Joos Kiener <Joos.Kiener@gmail.com>
 */
@Repository
@Transactional(propagation = Propagation.MANDATORY)
public interface ChemicalCompoundRepository<T extends ChemicalCompound>
        extends ChemicalStructureSearchRepository<T>, IdRepository<T>,
        BaseEntityRepository<T> {

    /**
     * Get all compounds which contained the chemical structure with id
     * <code>structureId</code>.
     *
     * @param structureId
     * @return
     */
    List<T> findByCompositionsPkChemicalStructureId(Long structureId);

    /**
     * Returns a compound by its CAS number. CAS field is unique hence single Result
     *
     *
     * @param cas
     * @return
     */
    T findByCas(String cas);
}