/*
 *  Copyright (C) 2013 Joos Kiener <Joos.Kiener@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.

 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bitbucket.kienerj.moleculedatabaseframework.repository;

import com.mysema.query.types.Predicate;
import com.mysema.query.types.path.PathBuilder;
import java.util.List;
import org.springframework.data.repository.NoRepositoryBean;

/**
 *<p>Repository that returns ids instead of full objects.</p>
 * <p>
 * Useful if a large number of objects must be selected and worked on like
 * when exporting to SD-File.
 * </p>
 *
 * @author Joos Kiener <Joos.Kiener@gmail.com>
 */
@NoRepositoryBean
public interface IdRepository<T> {

    /**
     * <p>Gets a list of ids for all matching entities.</p>
     *
     * @param predicate the WHERE clause / filter for this query
     * @param compoundPathBuilder defines the type of entity to search in
     * @return
     */
    List<Long> findAll(Predicate predicate,
            PathBuilder<T> compoundPathBuilder);

}
