/*
 * Copyright (C) 2013 Joos Kiener <Joos.Kiener@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bitbucket.kienerj.moleculedatabaseframework.service;

import com.google.common.base.Preconditions;
import com.mysema.query.types.OrderSpecifier;
import com.mysema.query.types.Predicate;
import java.util.HashMap;
import org.bitbucket.kienerj.moleculedatabaseframework.entity.Role;
import org.bitbucket.kienerj.moleculedatabaseframework.repository.RoleRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 *
 * @author Joos Kiener <Joos.Kiener@gmail.com>
 */
@Service("roleService")
@Transactional(readOnly=true)
public class RoleServiceImpl implements RoleService {

    @Autowired
    private RoleRepository roleRepository;

    @Override
    public final Role getById(Long id) {
        return this.roleRepository.findOne(id);
    }

    @Override
    public final Role findByRoleName(String roleName){
        return this.roleRepository.findByRoleName(roleName);
    }

    @Override
    public final Role findOne(Predicate predicate){
         return this.roleRepository.findOne(predicate);
    }

    @Override
    public final Iterable<Role> findAll(Predicate predicate){
         return this.roleRepository.findAll(predicate);
    }

    @Override
    public final Iterable<Role> findAll(Predicate predicate, OrderSpecifier<?>... orders){
         return this.roleRepository.findAll(predicate, orders);
    }

    @Override
    public final Page<Role> findAll(Predicate predicate, Pageable pageable){
         return this.roleRepository.findAll(predicate, pageable);
    }

    @Transactional(readOnly = false)
    @Override
    public final Role save(Role user){
         return this.roleRepository.save(user);
    }

    @Transactional(readOnly = false)
    @Override
    public final void delete(Role user){
        this.roleRepository.delete(user);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public UniquenesscheckResult checkUniqueness(Role role) {
        Preconditions.checkNotNull(role);
        Preconditions.checkNotNull(role.getRoleName());
        Role result = this.findByRoleName(role.getRoleName());

        HashMap<String, Object> fields = new HashMap<>();
        boolean isUnique = true;

        if (result != null){
            fields.put("roleName", role.getRoleName());
            isUnique = false;
        }
        return new UniquenesscheckResult(isUnique, fields);
    }

    @Override
    public Class getEntityClass(){
        return Role.class;
    }

}
