/*
 *  Copyright (C) 2013 Joos Kiener <Joos.Kiener@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.

 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bitbucket.kienerj.moleculedatabaseframework.service;

import org.bitbucket.kienerj.moleculedatabaseframework.entity.BaseEntity;

/**
 * To give all services a common type.
 *
 * @author Joos Kiener <Joos.Kiener@gmail.com>
 */
public interface Service<T extends BaseEntity> {

    /**
     * Get entity by its database id
     *
     */
    T getById(Long id);

    /**
     * Creates or updates <code>entity</code>.
     *
     * @param entity
     * @return
     */
    T save(T entity);

    /**
     * <p>
     * Checks if <code>entity</code> violates any unique constraints.
     * </p>
     * <p>
     * If <code>returnValue.isUnique() == true</code>, the entity can be safely
     * inserted. If false, a call to service.save(entity) will end in a
     * <code>DataIntegrityViolationException</code>.
     * </p>
     * <p>
     * The returned value contains a mapping of all fields an their values
     * that violate uniqueness.
     * </p>
     *
     * @param compound
     */
    UniquenesscheckResult checkUniqueness(T entity);

    /*
     * Gets the Class of the entity this service is for
     */
    Class getEntityClass();

}
