/*
 *  Copyright (C) 2013 Joos Kiener <Joos.Kiener@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.

 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bitbucket.kienerj.moleculedatabaseframework.service;

import com.mysema.query.types.OrderSpecifier;
import com.mysema.query.types.Predicate;
import org.bitbucket.kienerj.moleculedatabaseframework.entity.User;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * Service interface for <code>User</code> entity.
 * </p>
 * <p>
 * Extends  <code>UserDetailsService</code> and can hence be directly used in
 * Spring security for obtaining / checking credentials.
 * </p>
 *
 * @author Joos Kiener <Joos.Kiener@gmail.com>
 */
@Transactional(readOnly = true)
public interface UserService<T extends User> extends UserDetailsService, Service<T>  {

    //@PreAuthorize("hasRole('read_User')")
    @Override
    UserDetails loadUserByUsername(String username) throws UsernameNotFoundException;

    @PreAuthorize("hasRole('read_User')")
    T findByUsername(String username);

    @PreAuthorize("hasRole('read_User')")
    T findOne(Predicate predicate);

    @PreAuthorize("hasRole('read_User')")
    Iterable<T> findAll(Predicate predicate);

    @PreAuthorize("hasRole('read_User')")
    Iterable<T> findAll(Predicate predicate, OrderSpecifier<?>... orders);

    @PreAuthorize("hasRole('read_User')")
    Page<T> findAll(Predicate predicate, Pageable pageable);

    @Transactional(readOnly = false)
    @PreAuthorize("hasRole('manage_User')")
    @Override
    T save(T user);

    @Transactional(readOnly = false)
    @PreAuthorize("hasRole('manage_User')")
    void delete (T user);
}
