/*
 * Copyright (C) 2013 Joos Kiener <Joos.Kiener@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bitbucket.kienerj.moleculedatabaseframework.sql;

import java.util.Map;
import lombok.Getter;
import org.bitbucket.kienerj.moleculedatabaseframework.MoleculeDatabaseFrameworkException;

/**
 * <p>
 * Thrown when a entity is saved (insert or update) and a UniqueConstraintViolation
 * is detected.
 * </p>
 *
 * <p>
 * <code>getViolatedConstraints()</code> returns a Map of "constraint description"
 * and the offending value. "constraint description" is a either a field name or
 * a description of the constraint in case of a multi-field constraint.
 * </p>
 *
 * @author Joos Kiener <Joos.Kiener@gmail.com>
 */
public class UniqueConstraintViolationException extends MoleculeDatabaseFrameworkException {

    private final static String MESSAGE = "An entity violated one or more unique constraints";

    @Getter
    private final Map<String, Object> violatedContraints;

    public UniqueConstraintViolationException(Map<String, Object> violatedContraints, Throwable cause) {
        super(MESSAGE, cause);
        this.violatedContraints = violatedContraints;
    }

    public UniqueConstraintViolationException(String message,
            Map<String, Object> violatedContraints,
            Throwable cause) {
        super(message, cause);
        this.violatedContraints = violatedContraints;
    }
}
